import 'package:flutter/widgets.dart';
import 'package:ngex/store.dart';

class StoreBuilder<T extends StoreValue> extends StatefulWidget {
  const StoreBuilder({
    Key? key,
    required this.value,
    required this.builder,
  }) : super(key: key);

  final T value;
  final Widget Function(BuildContext context, T state) builder;

  @override
  StoreBuilderState<T> createState() => StoreBuilderState<T>();
}

class StoreBuilderState<T extends StoreValue> extends State<StoreBuilder<T>> {
  @override
  void initState() {
    widget.value.listen.addListener(_listener);
    super.initState();
  }

  @override
  void didUpdateWidget(covariant StoreBuilder<T> oldWidget) {
    if (widget.value != oldWidget.value) {
      _miggrate(widget.value.listen, oldWidget.value.listen, _listener);
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    widget.value.listen.removeListener(_listener);
    super.dispose();
  }

  void _miggrate(Listenable a, Listenable b, void Function() listener) {
    a.removeListener(listener);
    b.addListener(listener);
  }

  void _listener() {
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return widget.builder(context, widget.value);
  }
}
